@@ -1,34 +1,16 @@
+# -*- coding: utf-8 -*-
 
-import sys, os
+"""
+    sanescript.config
+    ~~~~~~~~~~~~~~~~~
 
-from argparse import ArgumentParser
+    Define option schemas, and read config data from a number of sources.
 
+    :copyright: 2009-2010 Ali Afshar <aafshar@gmail.com>
+    :license: MIT
+"""
 
-class _OptionProcessors(object):
-    """Some common processors
-    """
-    def python_import(self, import_name):
-        """Import a python string
-        """
-        if ':' in import_name:
-            module, obj = import_name.split(':', 1)
-        elif '.' in import_name:
-            module, obj = import_name.rsplit('.', 1)
-        else:
-            return __import__(import_name)
-        return getattr(__import__(module, None, None, [obj]), obj)
-
-    def file(self, mode='r'):
-        def _file(file_name, mode=mode):
-            return open(file_name, mode)
-        return _file
-
-    def virtualenv(self, path):
-        path = os.path.abspath(path)
-        activator = os.path.join(path, 'bin', 'activate_this.py')
-        execfile(activator, dict(__file__ = activator))
-
-processors = _OptionProcessors()
+import os
 
 
 class Unset(object):
@@ -41,6 +23,8 @@ class Unset(object):
     def __str__(self):
         return '<UNSET>'
 
+# We must have an UNSET state to differentiate from native types which provide
+# a None equivalent as valid data
 UNSET = Unset()
 
 
@@ -58,8 +42,12 @@ class Option(object):
                        name parameter.
     :param converter: a Python callable to pass the value into to convert to a
                       useful Python value.
+    :param processor: a Python callable used to process the data. This is
+                      distinct from the converter, because processors are
+                      applied to native types not needing conversion
     :param argparse_kw: additional keyword arguments to pass to the argparse
                         add_arg call.
+    :param positional: flag as to whether the argument is positional or not
 
     >>> o = Option('test', 'this is a test option', 'default')
     >>> o.long_opt
@@ -98,37 +86,24 @@ class Config(object):
     """
     def __init__(self, parser=None, env_prefix=None):
         self._env_prefix = env_prefix
-        self._parser = parser or ArgumentParser() # we don't always get passed
-                                                  # a parser because we may be
-                                                  # running independently of
-                                                  # the scripting framework.
+        self._parser = parser
         self._options = {}
 
-    def add_option(self, opt, prefix=None, parser=None):
+    def add_option(self, opt, prefix=None):
         """Add an option declaration.
 
         :param opt: an Option instance.
         """
         self._options[opt.name, prefix] = opt
-        parser = parser or self._parser
-        kw = opt.optparse_kw.copy()
-        if opt.converter:
-            kw['type'] = opt.converter
-        if opt.positional:
-            parser.add_argument(opt.name,
-                                help=opt.help, default=UNSET,
-                                **kw)
-        else:
-            parser.add_argument(opt.short_opt, opt.long_opt,
-                                help=opt.help, default=UNSET,
-                                **kw)
+        if prefix != "__pre__" and self._parser:
+            self._parser.add_option(opt, prefix=prefix)
         setattr(self, opt.name, opt.default)
 
-    def add_options(self, options, prefix=None, parser=None):
+    def add_options(self, options, prefix=None):
         """Add a number of options.
         """
         for option in options:
-            self.add_option(option, prefix, parser)
+            self.add_option(option, prefix)
 
     def grab_from_env(self, env=None):
         """Search the env dict for overriden values.
@@ -171,11 +146,17 @@ class Config(object):
     def grab_from_argv(self, argv):
         """Get values from argv list
         """
-        args = self._parser.parse_args(argv)
+        args = self._parser.parse(argv)
+        self.grab_from_ns(args)
+        try:
+            self.__command__ = args.__command__
+        except AttributeError:
+            self.__command__ = None
 
+    def grab_from_ns(self, ns):
         for (name, prefix), opt in self._options.items():
             try:
-                val = getattr(args, name)
+                val = getattr(ns, name)
                 if val is UNSET:
                     continue
             except AttributeError:
@@ -183,10 +164,6 @@ class Config(object):
             if opt.processor is not None:
                 val = opt.processor(val)
             setattr(self, name, val)
-        try:
-            self.__command__ = args.__command__
-        except AttributeError:
-            self.__command__ = None
 
     def grab_from_dict(self, d):
         """Load the options from a dict.
@@ -196,24 +173,6 @@ class Config(object):
         for k, v in d.items():
             setattr(self, k, v)
 
-    def grab_from_interactive(self, prefix, include, exclude):
-        if not include:
-            include = [n for (n, p) in self._options.keys()]
-        for name in include:
-            if name in exclude:
-                continue
-            opt = self._options.get((name, prefix))
-            if opt is None:
-                continue
-            from utils import do_prompt
-            raw = do_prompt(name, opt.help, opt.default or UNSET)
-            val = raw.strip()
-            if opt.converter:
-                val = opt.converter(val)
-            if opt.processor:
-                val = opt.processor(val)
-            setattr(self, name, val)
-
     def items(self):
         for (name, prefix) in self._options:
             yield name, getattr(self, name)
@@ -225,24 +184,12 @@ class Config(object):
         """
         return dict(self.items())
 
-    def ensure_required(self):
-        for (name, prefix), option in self._options.items():
-            if prefix != self.__command__:
-                continue
-            if option.required and getattr(self, option.name) is UNSET:
-                print 'Error: "%s" is required.' % option.name
-                print
-                print 'It could be provided as:'
-                print
-                print '\t-%s --%s on the command line,' % (option.short_name, option.name)
-                if prefix:
-                    print '\t%s_%s' % (prefix.upper(), name.upper()), 'in the environment'
-                else:
-                    print name, 'in the environment'
-                print '\tOr in any config files'
-                print
-                return False
-        return True
+    def pregrab_from_argv(self, argv):
+        """Internal Sanescript API to pregrab some vital information from
+           argparse before parsing the entire command line.
+        """
+        args = self._parser.pre_parse(argv)
+        self.grab_from_ns(args)
 
 
 class ConfigFile(object):
